/********************************************************************
 * Job Ticket API (JTAPI) fsgjt.h                                         
 *  
 * Copyright (c) 2004 High North Inc.                                                                  
 * Copyright (c) 2004 International Business Machines Corp.
 * Copyright (c) 2004 Seiko-Epson, Inc.
 * Copyright (c) 2004 Till Kamppeter.  
 * Copyright (c) 2004 Free Standards Group.                   
 *                                                                    
 *                    Common Public License - v 1.0 
 * THE ACCOMPANYING PROGRAM IS PROVIDED UNDER THE TERMS OF THIS 
 * COMMON PUBLIC LICENSE ("AGREEMENT"). ANY USE, REPRODUCTION OR 
 * DISTRIBUTION OF THE PROGRAM CONSTITUTES RECIPIENT'S ACCEPTANCE 
 * OF THIS AGREEMENT.   
 *
 * The Common Public License in its entirety is located in 
 * fsgjt_cpl.h and applies to the contents of this file. 
 *******************************************************************/
#ifndef FSGJT_ATTRIBUTE_H
#define FSGJT_ATTRIBUTE_H

#include <fsgjt.h>
#include <fsgjt_enums.h>

/************************************************************
 * fsgjtNewAttribute - Creates a new Attribute object having 
 * the provided attribute name, value type, and value. The
 * value must not be an extension value.
 * 
 * If this attribute is to contain an extension value, first 
 * instantiate the Attribute using newAttribute providing 
 * the appropriate FSGJT_xx_NOT_SET value, and then call 
 * fsgAddExtensionValue providing the value that is an 
 * extension.
 *
 * param (output) fsgjt_attribute_t* - Pointer to the new 
 *      attribute is returned. 
 * param (input) char* name - Name for the new attribute to 
 *      create
 * param (input) char* type - Type for the new attribute
 *      (use Value Types)
 * param (input) void* value - Pointer to the value for this 
 *      attribute. Multiple values should be added with the 
 *      add() function.
 *
 * return fsgjt_return_code_t
 ************************************************************/
fsgjt_return_code_t fsgjtNewAttribute(
               fsgjt_attribute_t* attribute, 
               char* name, 
               char* type, 
               void* value);

/************************************************************
 * fsgjtDestroyAttribute - Free the memory used by the 
 * Attribute.
 *
 * param (input) fsgjt_attribute_t attribute - Attribute to 
 *      free
 *
 * return fsgjt_return_code_t
 ************************************************************/
fsgjt_return_code_t fsgjtDestroyAttribute(
               fsgjt_attribute_t attribute);

/************************************************************
 * fsgjtAddExtensionValue - Add an additional value, that is 
 * an extension, to this Attribute. 
 *
 * param (input) fsgjt_attribute_t attribute - Attribute to
 *      add the provided extension value to 
 * param (input) char* extensionValue - Extension value to 
 *      add
 *
 * return fsgjt_return_code_t 
 ************************************************************/
fsgjt_return_code_t fsgjtAddExtensionValue(
               fsgjt_attribute_t attribute, 
               char* extensionValue);

/************************************************************
 * fsgjtAddValue - Add an additional value, that is not an 
 * extension, to this Attribute.
 *
 * param (input) fsgjt_attribute_t attribute - Attribute to
 *      add the provided value to 
 * param (input) void* value - Pointer to value, that is not
 *      and extension, to add
 *
 * return fsgjt_return_code_t 
 ************************************************************/
fsgjt_return_code_t fsgjtAddValue(
               fsgjt_attribute_t attribute, 
               void* value);

/************************************************************
 * fsgjtGetName - Get the name of the Attribute.
 *
 * param (input) fsgjt_attribute_t attribute - Attribute 
 *      whose name to get
 * param (output) char** name - Name of the attribute
 *
 * return fsgjt_return_code_t
 ************************************************************/
fsgjt_return_code_t fsgjtGetName(
               fsgjt_attribute_t attribute,
               char** name);

/************************************************************
 * fsgjtGetNextExtensionValue - Return the Attribute's next 
 * value that is an extension.
 *
 * param (input) fsgjt_attribute_t attribute - Attribute 
 *       whose next value is to be returned
 * param (output) char** nextExtensionValue - Pointer to the 
 *       next value that is an extension
 * param (output) fsgjt_present_t* moreValues - On, if there
 *       are more values for the Attribute other than the
 *       value being returned 
 *
 * return fsgjt_return_code_t 
 ************************************************************/
fsgjt_return_code_t fsgjtGetNextExtensionValue(
               fsgjt_attribute_t attribute,
               char** nextExtensionValue,
               fsgjt_present_t* moreValues);

/************************************************************
 * fsgjtGetNextValue - Return the Attribute's next value 
 * that is not an extension.
 *
 * param (input) fsgjt_attribute_t attribute - Attribute 
 *       whose next value is to be returned
 * param (output) void** nextValue - Pointer to the next 
 *       value that is not an extension
 * param (output) fsgjt_present_t* moreValues - On if there
 *       are more values for the Attribute other than the
 *       value being returned 
 *
 * return fsgjt_return_code_t 
 ************************************************************/
fsgjt_return_code_t fsgjtGetNextValue(
               fsgjt_attribute_t attribute,
               void** nextValue,
               fsgjt_present_t* moreValues);

/************************************************************
 * fsgjtGetNumExtensionValues - Get the number of values, 
 * that are extensions, that the Attribute contains.
 *
 * param (input) fsgjt_attribute_t attribute - Attribute 
 *      whose number of values to get
 * param (output) fsgjt_int32_t* numExtensionValues - Number 
 *      of values that are extensions
 *
 * return fsgjt_return_code_t
 ************************************************************/
fsgjt_return_code_t fsgjtGetNumExtensionValues(
               fsgjt_attribute_t attribute,
               fsgjt_int32_t* numExtensionValues);

/************************************************************
 * fsgjtGetNumValues - Get the number of values, that are not 
 * extensions, that the Attribute contains.
 *
 * param (input) fsgjt_attribute_t attribute - Attribute 
 *      whose number of values to get
 * param (output) fsgjt_int32_t* numValues - Number of 
 *      values that are not extensions
 *
 * return fsgjt_return_code_t
 ************************************************************/
fsgjt_return_code_t fsgjtGetNumValues(
               fsgjt_attribute_t attribute,
               fsgjt_int32_t* numValues);

/************************************************************
 * fsgjtGetValueType - Returns the type of the Attribute's 
 * values.
 *
 * param (input) fsgjt_attribute_t attribute - Attribute 
 *       whose value type to get
 * param (output) char** type - Type of this Attribute's 
 *       values (use Value Types)
 *
 * return fsgjt_return_code_t
 ************************************************************/
fsgjt_return_code_t fsgjtGetValueType(
               fsgjt_attribute_t attribute,
               char** type);

/************************************************************
 * fsgjtisExtensible - Returns true if this Attribute is
 * extensible, which means that a value other than a
 * pre-defined value can be added. Returns false if no value
 * other than a pre-defined value can be added.
 *
 * param (input) fsgjt_attribute_t attribute - Attribute to 
 *      check to see if it is extensible.
 * param (output) fsgjt_boolean_t* extensible - Pointer to 
 *      true if this attribute is extensible, otherwuise 
 *      false.
 *
 * return fsgjt_return_code_t
 ************************************************************/
fsgjt_return_code_t fsgjtisExtensible(
               fsgjt_attribute_t attribute,
               fsgjt_boolean_t* extensible);

/************************************************************
 * fsgjtReplaceExtensionValue - Replaces the existing 
 * extension value(s) for this Attribute with the provided 
 * value that is an extension. The non-extension value(s) of
 * this Attribute are not affected.
 * 
 * param (input) fsgjt_attribute_t attribute - Attribute to
 *      replace the provided value(s)  
 * param (input) char* extensionValue - Replacement value 
 *      that is an extension
 *
 * return fsgjt_return_code_t 
 ************************************************************/
fsgjt_return_code_t fsgjtReplaceExtensionValue(
               fsgjt_attribute_t attribute, 
               char* extensionValue);

/************************************************************
 * fsgjtReplaceValue - Replaces the existing non-extension
 * value(s) for this Attribute with the provided value that is
 * not an extension. The extension value(s) of this Attribute  
 * are not affected.
 * 
 * param (input) fsgjt_attribute_t attribute - Attribute to
 *      replace the provided value(s)  
 * param (input) void* value - Pointer to replacement value
 *      that is not an extension
 *
 * return fsgjt_return_code_t 
 ************************************************************/
fsgjt_return_code_t fsgjtReplaceValue(
               fsgjt_attribute_t attribute, 
               void* value);

/************************************************************
 * fsgjtResetToFirstValue - Reset the iterators to point to 
 * the first non-extension value and the first extension 
 * value of the Attribute.
 *
 * param (input) fsgjt_attribute_t attribute - Attribute 
 *      to reset
 *
 * return fsgjt_return_code_t
 ************************************************************/
fsgjt_return_code_t fsgjtResetToFirstValue(
               fsgjt_attribute_t attribute);

#endif /* FSGJT_ATTRIBUTE_H */
